Attribute VB_Name = "NT4RASPerf"
Option Explicit

 ' perf declarations

Public Type LARGE_INTEGER
   lowpart As Long
   highpart As Long
End Type

Public Type SYSTEMTIME
        wYear As Integer
        wMonth As Integer
        wDayOfWeek As Integer
        wDay As Integer
        wHour As Integer
        wMinute As Integer
        wSecond As Integer
        wMilliseconds As Integer
End Type


Public Type PERF_DATA_BLOCK
    Signature(7) As Byte      ' Signature: Unicode "PERF"
    LittleEndian As Long
    Version As Long
    Revision As Long
    TotalByteLength As Long    ' Total length of data block
    HeaderLength As Long       ' Length of this structure
    NumObjectTypes As Long     ' Number of types of objects  being reported
    DefaultObject As Long      ' Object Title Index of default object to display
    SysTime As SYSTEMTIME      'Time at the system under measurement
    PerfTime As LARGE_INTEGER  ' Performance counter value at the system under measurement
    PerfFreq As LARGE_INTEGER  ' Performance counter frequency at the system under measurement
    PerfTime100nSec As LARGE_INTEGER  'Performance counter time in 100 nsec units at the system under measurement
    paddingBytes As Long      ' fixes alignment of this structure ??
    SystemNameLength As Long    'Length of the system name
    SystemNameOffset As Long    ' Offset, from beginning of this structure, to name of system being measured
End Type

Public Type PERF_OBJECT_TYPE
   TotalByteLength As Long      'offset to the next object
   DefinitionLength As Long     ' offset to data or instancedef
   HeaderLength  As Long        'offset to the first counterdef
   ObjectNameTitleIndex As Long 'Index to name in Title Database
   ObjectNameTitle As Long      'Initially NULL
   ObjectHelpTitleIndex As Long 'Index to Help in Title Database
   ObjectHelpTitle As Long      'Initially NULL, for use by
   DetailLevel As Long          'Object level of detail
   NumCounters As Long          'Number of counters in each counter block
   DefaultCounter As Long       'Default counter to display
   NumInstances As Long         'Number of object instances
   CodePage As Long             '0 if instance strings are in UNICODE
   PerfTime As LARGE_INTEGER    'Sample Time in "Object" units
   PerfFreq As LARGE_INTEGER    'Frequency of "Object" units in counts per second.
End Type

Public Type PERF_COUNTER_DEFINITION
    ByteLength As Long            'Length in bytes of this structure
    CounterNameTitleIndex As Long 'Index of Counter name into Title Database
    CounterNameTitle As Long      'Initially NULL
    CounterHelpTitleIndex As Long 'Index of Counter Help into Title Database
    CounterHelpTitle As Long      'Initially NULL
    DefaultScale As Long          'Power of 10 by which to scale
    DetailLevel As Long           'Counter level of detail
    CounterType As Long           'Type of counter
    CounterSize As Long           'Size of counter in bytes
    CounterOffset As Long         'Offset from the start to counter's value
End Type

Public Type PERF_INSTANCE_DEFINITION
    ByteLength As Long         'Length in bytes of this structure including the subsequent name
    ParentObjectTitleIndex As Long 'Title Index to name of "parent"
    ParentObjectInstance As Long 'Index to instance of parent object
    UniqueID As Long
    NameOffset As Long          'Offset from beginning of this struct to the Unicode name
    NameLength As Long         'Length in bytes of name; 0 = none
End Type


Public Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" _
                 (Destination As Any, Source As Any, ByVal Length As Long)


' * REGISTRY declarations *

Public Declare Function RegCloseKey _
      Lib "advapi32.dll" _
      (ByVal hKey As Long) As Long

Public Declare Function RegOpenKeyEx _
      Lib "advapi32.dll" Alias "RegOpenKeyExA" _
      (ByVal hKey As Long, _
      ByVal lpSubKey As String, _
      ByVal ulOptions As Long, _
      ByVal samDesired As Long, _
      phkResult As Long) As Long

Public Declare Function RegQueryValueEx _
      Lib "advapi32.dll" Alias "RegQueryValueExA" _
      (ByVal hKey As Long, _
      ByVal lpValueName As String, _
      ByVal lpReserved As Long, _
      lpType As Long, _
      lpData As Any, _
      lpcbData As Long) As Long

Public Const HKEY_LOCAL_MACHINE = &H80000002
Public Const HKEY_PERFORMANCE_DATA = &H80000004
Public Const KEY_READ = &H20019
Public Const ERROR_MORE_DATA = 234


'***** UDTs to store data in ****

Public Type VBRasStatCounter
   strName As String
   lngNameIndex As Long
   lngValueType As Long
   varValue As Variant
   base10 As Long
End Type

Public Type VBRasStatInstance
   strName As String
   typCounters() As VBRasStatCounter
End Type

Public Type VBRASStatObject
   strName As String
   lngNameIndex As Long
   typInstances() As VBRasStatInstance
End Type



Dim rtn As Long, lngHkey As Long
Dim lngEnd As Long, lngStart As Long
Dim lpcbData As Long, strCounters As String
Dim strId As String, strTemp As String

' note: this uses the strCounters from the code above
Dim i As Long
Dim astrNames() As String
Dim colNames As New Collection

Dim b() As Byte
Dim lngIncr As Long


Dim prfDataBlock As PERF_DATA_BLOCK
Dim prfObjectDef As PERF_OBJECT_TYPE
Dim prfCounterDef As PERF_COUNTER_DEFINITION
Dim prfInstanceDef As PERF_INSTANCE_DEFINITION
Dim nObjects As Long ' number of objects
Dim nObjIdx As Long  ' used to loop through objects
Dim nInsts As Long ' number of instances
Dim nCntrs As Long ' number of counters
Dim iPos As Long ' marker for b()
Dim iNextObjPos As Long 'marker for next object in b()
Dim aValuePos() As Long 'markers for where the values start
Dim j As Long, k As Long, n As Long 'used for iteration
'Dim i as long 'should have already been dimmed.
'Dim strTemp As String 'should have already been dimmed.




Dim c As Long, ic As Long

Dim btj As Long, brj As Long, bi As Long


' diese Funktion ist viel zu langsam, um 1x pro Sekunde
' aufgerufen zu werden
' sicher mu das meisste nur einmal am Anfang gemacht werden,
' aber ich sehe hier einfach nicht durch...
'
Function getNT4RASTraffic() As Long

'Debug.Print "getNT4RASTraffic()"

If Not RAS_NT4PerfIsInitialized Then
  Call InitNT4RASPerf
  RAS_NT4PerfIsInitialized = True
End If


Do
   lngIncr = lngIncr + 2048
   lpcbData = lngIncr
   ReDim b(lpcbData - 1)
   rtn = RegQueryValueEx(HKEY_PERFORMANCE_DATA, strId, 0&, _
                                    ByVal 0&, b(0), lpcbData)

Loop While rtn = ERROR_MORE_DATA

rtn = RegCloseKey(HKEY_PERFORMANCE_DATA)


'-------------------------------------------------------------------------------------


'------------------------------------------------------------------------------

'4. Crunching the bytes

Dim typPerfObject() As VBRASStatObject

 '***** Crunch those bytes ******
'Get the number of objects & pointer to first object
CopyMemory prfDataBlock, b(0), LenB(prfDataBlock)
nObjects = prfDataBlock.NumObjectTypes
iPos = prfDataBlock.HeaderLength
' check to see that are some objects
If nObjects < 1 Then Exit Function
nObjects = nObjects - 1 ' zero base our array
ReDim typPerfObject(nObjects)

'start our outer loop of objects
For nObjIdx = 0 To nObjects
'get the first object
   CopyMemory prfObjectDef, b(iPos), LenB(prfObjectDef)
   
   iNextObjPos = iPos + prfObjectDef.TotalByteLength
   nInsts = prfObjectDef.NumInstances
   nCntrs = prfObjectDef.NumCounters - 1
   If nInsts = -1 Then nInsts = 1
   nInsts = nInsts - 1 ' zero base our array
   
   On Error GoTo NextObject
   ReDim typPerfObject(nObjIdx).typInstances(nInsts)
   With typPerfObject(nObjIdx)
      For i = 0 To nInsts
         ReDim .typInstances(i).typCounters(nCntrs)
      Next i
   End With
   
   ' On Error Resume Next
   'get our object's name
   With typPerfObject(nObjIdx)
      .lngNameIndex = prfObjectDef.ObjectNameTitleIndex
      .strName = colNames(CStr(.lngNameIndex))
   End With
   
   'get some markers to retrieve values
   ReDim aValuePos(nInsts)
   If prfObjectDef.NumInstances = -1 Then
      'no instance def blocks
      aValuePos(0) = iPos + prfObjectDef.DefinitionLength
      With typPerfObject(nObjIdx)
         .typInstances(0).strName = .strName
      End With
   Else
      j = iPos + prfObjectDef.DefinitionLength
      'get instances names and data offsets
      For i = 0 To nInsts
         CopyMemory prfInstanceDef, b(j), LenB(prfInstanceDef)
         
         'get name of instance while we're here
         k = prfInstanceDef.NameLength - 2
         If k > 0 Then
            strTemp = String(k \ 2, 0)
            CopyMemory ByVal StrPtr(strTemp), _
                 b(j + prfInstanceDef.NameOffset), k
            typPerfObject(nObjIdx).typInstances(i).strName = strTemp
         End If
      
         'get the offset for each values block
         j = j + prfInstanceDef.ByteLength
         aValuePos(i) = j
         'find the next instancedef block
         ' first four bytes tells us length of value block
         ' which is followed by next instance def (if any)
         CopyMemory k, b(j), 4
         j = j + k
      Next i
   End If

  Dim BT As Long, BR As Long
  BT = -1
  BR = -1
  
   ' put our iPos to the first CounterDef block
   iPos = iPos + prfObjectDef.HeaderLength
   For i = 0 To nCntrs
      CopyMemory prfCounterDef, b(iPos), LenB(prfCounterDef)
        For j = 0 To nInsts
           With typPerfObject(nObjIdx).typInstances(j).typCounters(i)
              .lngNameIndex = prfCounterDef.CounterNameTitleIndex
              .strName = colNames(CStr(.lngNameIndex))
              .base10 = prfCounterDef.DefaultScale
              .lngValueType = prfCounterDef.CounterType
              'normally we would check the countertype
              ' and decide if it's a string or byte array
              ' or Long, but all of RAS is long, so we'll
              ' only cover longs here
              If (.lngValueType And &H100) = False Then
                 If (.lngValueType And &H200) = False Then
                    CopyMemory k, _
                       b(aValuePos(j) + prfCounterDef.CounterOffset), 4
                    .varValue = k
                 End If
              End If
              If .strName = "Bytes Transmitted" Then
                BT = .varValue
              End If
              If .strName = "Bytes Received" Then
                BR = .varValue
                'Debug.Print BR
              End If
              'If BR >= 0 And BT >= 0 Then GoTo BrutalRaus
           End With
        Next j
      iPos = iPos + prfCounterDef.ByteLength
   Next i
  

NextObject:
   iPos = iNextObjPos
Next nObjIdx


BrutalRaus:
getNT4RASTraffic = BT + BR
'Debug.Print getNT4RASTraffic

'-------------------------------------------------------------------------


End Function


Sub InitNT4RASPerf()
Debug.Print "InitNT4RASPerf()"

bi = -1

Do Until colNames.Count = 0
  Call colNames.Remove(1)
Loop


rtn = RegOpenKeyEx(HKEY_LOCAL_MACHINE, _
       "SOFTWARE\Microsoft\Windows NT\CurrentVersion\Perflib\009", _
       0&, KEY_READ, lngHkey)


rtn = RegQueryValueEx(lngHkey, "Counter", 0&, ByVal 0&, ByVal 0&, lpcbData)

strCounters = String(lpcbData, 0)

rtn = RegQueryValueEx(lngHkey, "Counter", 0&, ByVal 0&, _
                                ByVal strCounters, lpcbData)

rtn = RegCloseKey(lngHkey)

lngEnd = InStr(3, strCounters, Chr$(0) & "RAS Port" & Chr$(0))
If lngEnd Then
   lngStart = InStrRev(strCounters, Chr$(0), lngEnd - 1)
   strId = Mid$(strCounters, lngStart + 1, lngEnd - lngStart - 1)
End If

lngEnd = InStr(3, strCounters, Chr$(0) & "RAS Total" & Chr$(0))
If lngEnd Then
   lngStart = InStrRev(strCounters, Chr$(0), lngEnd - 1)
   strTemp = Mid$(strCounters, lngStart + 1, lngEnd - lngStart - 1)
   If strId <> "" Then
      strId = strId & " " & strTemp
   Else
      strId = strTemp
   End If
End If

'---------------------------------------------------------------------------------




' 2. creating a counter names  collection


ic = InCount(strCounters, Chr$(0))

ReDim astrNames(ic)
c = 1
For i = 0 To ic - 1
  astrNames(i) = ParseString(c, strCounters, Chr$(0))
  'Debug.Print i, astrNames(i)
Next i

'astrNames = Split(strCounters, Chr$(0))

'On Error Resume Next

For i = 2 To UBound(astrNames) Step 2
   colNames.Add astrNames(i + 1), astrNames(i)
Next i



'---------------------------------------------------------------------------------



End Sub


'Nachbau der gleichnamigen QuickPack Funktion
Function ParseString$(ByRef CurPos As Long, ByVal Work As String, ByVal Delimit As String)

  Dim s As String
  Dim p As Long
  
  If Work = "" Then GoTo raus
  
  If CurPos = 0 Then
    GoTo raus
  End If
  
  p = InStr(CurPos, Work, Delimit)
  
  If p = 0 Then
    s = Mid$(Work, CurPos)
  Else
    s = Mid$(Work, CurPos, p - CurPos)
  End If
  
  
  CurPos = CurPos + Len(s) + Len(Delimit)
  
  ParseString$ = Trim$(s)

'Debug.Print ParseString$

raus:

End Function




'Nachbau der QuickPack-Function
Function InCount(pstr1 As String, pstrSearchStr As String) As Long

Dim lngStart As Long
Dim lngCount As Long

'on error GoTo lblError

lngStart = 1
lngCount = 0

lngStart = InStr(lngStart, pstr1, pstrSearchStr)
Do While lngStart > 0
  lngCount = lngCount + 1
  lngStart = InStr(lngStart + 1, pstr1, pstrSearchStr)
Loop

InCount = lngCount

Exit Function

lblError:
InCount = 0
Exit Function

End Function


' A slow and inefficient version
Function InStrRev(sTarget As String, _
                          sFind As String, _
                          Optional ByVal iStart As Long = -1, _
                          Optional ByVal vcmCompare As VbCompareMethod) As Long
            
    ' Handle missing arguments
    If iStart = -1 Then iStart = Len(sTarget)
    
    ' Search backward
    Dim cFind As Long, i As Long, f As Long
    cFind = Len(sFind)
    For i = iStart - cFind + 1 To 1 Step -1
        If StrComp(Mid$(sTarget, i, cFind), sFind, vcmCompare) = 0 Then
            InStrRev = i
            Exit Function
        End If
    Next
End Function

' Filter for VB5
Function Filter(vInput As Variant, sMatch As String, _
                Optional fInclude As Boolean = True, _
                Optional Compare As VbCompareMethod = vbBinaryCompare _
                ) As Variant
    Dim asRet() As String, c As Long, i As Long, s As String
    On Error GoTo FilterResize
    For i = 0 To UBound(vInput)
        s = vInput(i)
        If InStr(1, s, sMatch, Compare) Then
            If fInclude Then
                asRet(c) = s
                c = c + 1
            End If
        Else
            If Not fInclude Then
                asRet(c) = s
                c = c + 1
            End If
        End If
    Next
    ReDim Preserve asRet(0 To c - 1)
    Filter = asRet
    Exit Function
    
FilterResize:
    Const cChunk As Long = 20
    If Err.Number = 1 Then 'eeOutOfBounds
        ReDim Preserve asRet(0 To c + cChunk) As String
        Resume              ' Try again
    End If
    Err.Raise Err.Number     ' Other VB error for client
End Function

' Define the mighty Join function for VB5
Function Join(vaSourceArray As Variant, _
              Optional vsDelimiter As Variant) As String
    Dim i As Long
    If IsMissing(vsDelimiter) Then vsDelimiter = " "
    For i = LBound(vaSourceArray) To UBound(vaSourceArray) - 1
        Join = Join & vaSourceArray(i) & vsDelimiter
    Next
    Join = Join & vaSourceArray(i)
End Function


' Hey, kids! Write your very own VB6 functions for VB5! How about
' LoadPicture? Should be easy to fake (and do a better job) using
' the API LoadImage. See LoadAnyPicture for an example. Here are some
' others:
'
'   FormatCurrency (could be messy)
'   FormatDateTime (a challenge to make it international)
'   FormatNumber (you could do better than they did)
'   FormatPercent (similar to FormatNumber)
'   MonthName (easy at the first level, but try making it international)
'   Replace (interesting and useful project)
'   Round (fun challenge)
'   StrReverse (yes, but why?)
'   WeekdayName (same as MonthName)





